package labelrendering;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import javax.swing.JPanel;

/**
 * Test canvas for label rendering
 * @author ubik
 */
class LabelCanvas extends JPanel {

    private LabelRenderer lbRndr;
    private ArrayList<Street> streets = new ArrayList<Street>();
    private boolean createdLabels = false;
    
    public LabelCanvas() throws Exception {
        ClassLoader ldr = LabelCanvas.class.getClassLoader();
        Font fnt = Font.createFont(Font.TRUETYPE_FONT, ldr.getResourceAsStream("assets/Ubuntu-B.ttf"));
        //Font fnt = Font.createFont(Font.TRUETYPE_FONT, ldr.getResourceAsStream("assets/Ubuntu-R.ttf"));
        lbRndr = new LabelRenderer(fnt.deriveFont(40f));
        
        // /*
        streets.add( new Street("Elm Street",20.0).add(0, 50).add(100,70).add(200,150).add(300,150).add(300,300).add(500,300) );
        streets.add( new Street("Spooner Street",7.0).add(0,300).add(250,250).add(500,250));
        streets.add( new Street("Baker Street",7.0).add(400,0).add(400,500));
        streets.add( new Street("Chtulhu Rd.",7.0).add(0,310).add(250,310).add(250,500) );
        streets.add( new Street("Yellow Brick Road",7.0).add(0,350).add(200,350) );
        // */
        
        /* 
        lbRndr.addLabel("Betneans County", new Point2D.Float(150, 150), 0.0, .75);
        lbRndr.addLabel("Soapware Products", new Point2D.Float(250, 250), Math.PI * 0.5, .75);
        lbRndr.addLabel("Toxic Slush", new Point2D.Float(350, 350), Math.PI * 0.15, .5);
        lbRndr.addLabel("Slurm!", new Point2D.Float(100, 400), Math.PI * -0.15, .5);
        // */
        
        invalidate();
        repaint();
    }
    
    @Override
    public void paintComponent( Graphics grphcs ) {
        Graphics2D g = (Graphics2D) grphcs;
        if( !createdLabels ) {
            // A little ugly, but we need the font render context
            for (Street street : streets) {
                street.createLabels(lbRndr, g);
            }
            createdLabels = true;
        }
        
        Dimension size = getSize();
        g.setColor(new Color(0x666666));
        g.fillRect(0, 0, size.width, size.height);
        
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);                
        // g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);                
        g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_GASP);                
        
        g.setStroke(new BasicStroke(2.0f, BasicStroke.JOIN_ROUND, BasicStroke.CAP_ROUND));
        for (Street street : streets) {
            g.setColor(new Color(0x000000));
            street.paint(g, false);
        }        
        
        g.setStroke(new BasicStroke());
        for (Street street : streets) {
            g.setColor(new Color(0xffffaa));
            street.paint(g, true);
        }
        
        lbRndr.paint(g);
    }
}
