package labelrendering;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.RenderingHints;
import java.awt.TexturePaint;
import java.awt.font.FontRenderContext;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

/**
 *
 * @author ubik
 */
public class LabelRenderer {
    
    private ArrayList<Label> labels = new ArrayList<Label>();
    private Font font;
    
    public LabelRenderer( Font font ) {
        this.font = font;
    }
    
    public void addLabel( String text, Point2D position, double angle, double scale ) {
        labels.add(new Label(text, position, angle, scale));
    }
    
    public double getLabelLength( FontRenderContext cntxt, String text, double scale ) {
        Rectangle2D r = font.getStringBounds(text, cntxt);
        return r.getWidth() * scale;
    }
    
    public void paint( Graphics2D g ) {
        FontMetrics fm = g.getFontMetrics(font);
        Paint paint = g.getPaint();
        g.setFont(font);        
        for (Label label : labels) {
            // Debug: mark position
            /*
            int x = (int) label.x;
            int y = (int) label.y;
            g.setColor(Color.red);
            g.drawLine(x-2, y, x+2, y);
            g.drawLine(x, y-2, x, y+2);
            */
            // Obtain text bounds wrt current context
            Rectangle2D bnds = fm.getStringBounds(label.text, g);
            
            AffineTransform old = g.getTransform();            
            
            AffineTransform trns = new AffineTransform();
            trns.translate(label.x, label.y);
            trns.rotate(label.angle);          
            trns.scale(label.scale, label.scale);            
            // trns.translate(-bnds.getWidth()*.5, bnds.getHeight()*.5);
            trns.translate(-bnds.getWidth()*.5, fm.getHeight()*.33);
            g.transform(trns);
                    
            float f = 1.5f;
            g.setColor(Color.WHITE);            
            g.drawString(label.text, -f, -f);
            g.drawString(label.text, +f, -f);
            g.drawString(label.text, -f, +f);
            g.drawString(label.text, +f, +f);
            
            g.setColor(Color.BLACK); 
            g.drawString(label.text, 0, 0);
            
            g.setTransform(old);
        }
        g.setPaint(paint);
    }
    
    /**
     * SDO for label
     */
    private static class Label {
        protected final String text;
        protected final float x, y;
        protected final double angle;
        protected final double scale;
        
        public Label( String text, Point2D position, double angle, double scale ) {
            this.text = ""+text;
            this.angle = angle;            
            this.scale = scale;
            x = (float) position.getX();
            y = (float) position.getY();
        }
    }
}
