package labelrendering;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

/**
 *
 * @author ubik
 */
public class Street {
    private String name;
    private ArrayList<Point2D.Double> points = new ArrayList<Point2D.Double>();
    private double radius;
    
    public Street( String name, double radius ) {
        this.radius = radius;
        this.name = name;
    }
    
    public Street add(double x, double y) {
        points.add(new Point2D.Double(x, y));
        return this;
    }
    
    public void createLabels( LabelRenderer lbrndr, Graphics2D g ) {
        Double[] pnts = points.toArray(new Point2D.Double[0]);
        double s = .5;
        if( radius < 10 )
            s = .4;
        
        for (int i = 0; i < pnts.length-1; i++) {
            double dx = pnts[i+1].x-pnts[i].x;
            double dy = pnts[i+1].y-pnts[i].y;
            double angle = Math.atan2(dy, dx);
                        
            double dist = Math.sqrt(dx*dx+dy*dy);   
            assert( dist > 0.0 );
            
            double dirx = dx/dist;            
            double diry = dy/dist;            
            
            double len = lbrndr.getLabelLength(g.getFontRenderContext(), name, s);
            if( dist >= len*1.2f ) {
                Point2D.Double pos = new Point2D.Double(pnts[i].x,pnts[i].y);
                int num = (int)(dist/(len*1.2f));
                num = num <= 0 ? 1 : num;
                double width = dist/num;
                pos.x += dirx*width*.5;
                pos.y += diry*width*.5;
                
                for (int j = 0; j < num; j++) {
                    lbrndr.addLabel(name, pos, angle, s);                    
                    pos.x += dirx*width;
                    pos.y += diry*width;                    
                }
            }
        }
    }
        
    public void paint(Graphics2D g, boolean fill) {
        Double[] pnts = points.toArray(new Point2D.Double[0]);
        for (int i = 0; i < pnts.length-1; i++) {
            double dx = pnts[i+1].x-pnts[i].x;
            double dy = pnts[i+1].y-pnts[i].y;
            
            double len = Math.sqrt(dx*dx+dy*dy);
            double angle = Math.atan2(dy, dx);
            
            Rectangle2D.Double rect = new Rectangle2D.Double(0,0,len, 2*radius);
            Ellipse2D.Double jnt = new Ellipse2D.Double(-radius, -radius, 2*radius, 2*radius);

            AffineTransform trns = g.getTransform();
            g.translate(pnts[i].x, pnts[i].y);
            if( fill )
                g.fill(jnt);
            else
                g.draw(jnt);
                        
            g.rotate(angle);
            g.translate(0,-radius);
            
            if (fill)
                g.fill(rect);
            else
                g.draw(rect);
            
            g.setTransform(trns);
        }
    }
}
